package com.company.surveypoc.screen.survey;

import com.company.surveypoc.entity.SurveyQuestion;
import com.company.surveypoc.screen.surveyquestion.SurveyQuestionEdit;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.action.Action;
import io.jmix.ui.component.Table;
import io.jmix.ui.model.CollectionPropertyContainer;
import io.jmix.ui.screen.*;
import com.company.surveypoc.entity.Survey;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.Objects;

@UiController("Survey.edit")
@UiDescriptor("survey-edit.xml")
@EditedEntityContainer("surveyDc")
public class SurveyEdit extends StandardEditor<Survey> {
    @Autowired
    private CollectionPropertyContainer<SurveyQuestion> questionsDc;
    @Autowired
    private Table<SurveyQuestion> questionsTable;
    @Autowired
    private ScreenBuilders screenBuilders;

//    @Install(to = "questionsTable.create", subject = "screenConfigurer")
//    private void questionsTableCreateScreenConfigurer(final Screen screen) {
//        ((SurveyQuestionEdit) screen).setDisplayOrder(questionsDc.getItems().size() + 1);
//    }

    @Subscribe("questionsTable.create")
    public void onQuestionsTableCreate(final Action.ActionPerformedEvent event) {
        screenBuilders.editor(SurveyQuestion.class, this)
                .withScreenClass(SurveyQuestionEdit.class)
                .newEntity()
                .withInitializer(surveyQuestion -> {
                    surveyQuestion.setSurvey(getEditedEntity());
                    surveyQuestion.setDisplayOrder(questionsDc.getItems().size() + 1);
                })
                .withAfterCloseListener(screenCloseEvent -> {
                    if (screenCloseEvent.closedWith(StandardOutcome.COMMIT)) {
                        questionsDc.getMutableItems().add(screenCloseEvent.getSource().getEditedEntity());
                    }
                })
                .build()
                .show();
    }

    @Install(to = "questionsTable.moveUp", subject = "enabledRule")
    private boolean questionsTableMoveUpEnabledRule() {
        SurveyQuestion surveyQuestion = questionsTable.getSingleSelected();

        return Objects.requireNonNull(surveyQuestion).getDisplayOrder() > 1;
    }

    @Subscribe("questionsTable.moveUp")
    public void onQuestionsTableMoveUp(final Action.ActionPerformedEvent event) {
        SurveyQuestion question = questionsTable.getSingleSelected();
        Integer currentItemOrdinal = Objects.requireNonNull(question).getDisplayOrder();
        Integer targetOrdinal = currentItemOrdinal - 1;
        List<SurveyQuestion> surveyQuestions = questionsDc.getMutableItems();
        for (SurveyQuestion surveyQuestion : surveyQuestions) {
            if (surveyQuestion.getDisplayOrder().equals(targetOrdinal)) {
                surveyQuestion.setDisplayOrder(currentItemOrdinal);
            }
        }
        question.setDisplayOrder(targetOrdinal);
        questionsTable.sort("displayOrder", Table.SortDirection.ASCENDING);
    }

    @Install(to = "questionsTable.moveDown", subject = "enabledRule")
    private boolean questionsTableMoveDownEnabledRule() {
        SurveyQuestion surveyQuestion = questionsTable.getSingleSelected();

        return Objects.requireNonNull(surveyQuestion).getDisplayOrder() < questionsDc.getItems().size();
    }

    @Subscribe("questionsTable.moveDown")
    public void onQuestionsTableMoveDown(final Action.ActionPerformedEvent event) {
        SurveyQuestion question = questionsTable.getSingleSelected();
        Integer currentItemOrdinal = Objects.requireNonNull(question).getDisplayOrder();
        Integer targetOrdinal = currentItemOrdinal + 1;
        List<SurveyQuestion> surveyQuestions = questionsDc.getMutableItems();
        for (SurveyQuestion surveyQuestion : surveyQuestions) {
            if (surveyQuestion.getDisplayOrder().equals(targetOrdinal)) {
                surveyQuestion.setDisplayOrder(currentItemOrdinal);
            }
        }
        question.setDisplayOrder(targetOrdinal);
        questionsTable.sort("displayOrder", Table.SortDirection.ASCENDING);
    }
}