package com.company.surveypoc.screen.surveyquestion;

import com.company.surveypoc.entity.SurveyAnswerOption;
import com.company.surveypoc.entity.SurveyAnswerTypeEnum;
import com.company.surveypoc.screen.surveyansweroption.SurveyAnswerOptionEdit;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.action.Action;
import io.jmix.ui.component.GroupBoxLayout;
import io.jmix.ui.component.HasValue;
import io.jmix.ui.component.Table;
import io.jmix.ui.model.CollectionPropertyContainer;
import io.jmix.ui.screen.*;
import com.company.surveypoc.entity.SurveyQuestion;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.Objects;

@UiController("SurveyQuestion.edit")
@UiDescriptor("survey-question-edit.xml")
@EditedEntityContainer("surveyQuestionDc")
public class SurveyQuestionEdit extends StandardEditor<SurveyQuestion> {
    @Autowired
    private CollectionPropertyContainer<SurveyAnswerOption> answerOptionsDc;
    @Autowired
    private Table<SurveyAnswerOption> answerOptionsTable;
    @Autowired
    private GroupBoxLayout answerOptionsBox;
    @Autowired
    private ScreenBuilders screenBuilders;
    private Integer displayOrder;

//    @Subscribe
//    public void onInitEntity(final InitEntityEvent<SurveyQuestion> event) {
//        event.getEntity().setDisplayOrder(this.displayOrder);
//    }

    @Subscribe
    public void onBeforeShow(final BeforeShowEvent event) {
        showHideAnswerOptions();
    }

    @Subscribe("answerTypeField")
    public void onAnswerTypeFieldValueChange(final HasValue.ValueChangeEvent<SurveyAnswerTypeEnum> event) {
        showHideAnswerOptions();
    }

//    @Install(to = "answerOptionsTable.create", subject = "screenConfigurer")
//    private void answerOptionsTableCreateScreenConfigurer(final Screen screen) {
//            ((SurveyAnswerOptionEdit) screen).setDisplayOrder(answerOptionsDc.getItems().size() + 1);
//    }

    @Subscribe("answerOptionsTable.create")
    public void onAnswerOptionsTableCreate(final Action.ActionPerformedEvent event) {
        screenBuilders.editor(SurveyAnswerOption.class, this)
                .withScreenClass(SurveyAnswerOptionEdit.class)
                .newEntity()
                .withInitializer(answerOption -> {
                    answerOption.setSurveyQuestion(getEditedEntity());
                    answerOption.setDisplayOrder(answerOptionsDc.getItems().size() + 1);
                })
                .withAfterCloseListener(screenCloseEvent -> {
                    if (screenCloseEvent.closedWith(StandardOutcome.COMMIT)) {
                        answerOptionsDc.getMutableItems().add(screenCloseEvent.getSource().getEditedEntity());
                    }
                })
                .build()
                .show();
    }

    @Install(to = "answerOptionsTable.moveUp", subject = "enabledRule")
    private boolean answerOptionsTableMoveUpEnabledRule() {
        SurveyAnswerOption answerOption = answerOptionsTable.getSingleSelected();

        return Objects.requireNonNull(answerOption).getDisplayOrder() > 1;
    }

    @Subscribe("answerOptionsTable.moveUp")
    public void onAnswerOptionsTableMoveUp(final Action.ActionPerformedEvent event) {
        SurveyAnswerOption answerOption = answerOptionsTable.getSingleSelected();
        Integer currentItemOrdinal = Objects.requireNonNull(answerOption).getDisplayOrder();
        Integer targetOrdinal = currentItemOrdinal - 1;
        List<SurveyAnswerOption> surveyAnswerOptions = answerOptionsDc.getMutableItems();
        for (SurveyAnswerOption surveyAnswerOption : surveyAnswerOptions) {
            if (surveyAnswerOption.getDisplayOrder().equals(targetOrdinal)) {
                surveyAnswerOption.setDisplayOrder(currentItemOrdinal);
            }
        }
        answerOption.setDisplayOrder(targetOrdinal);
        sortTable();
    }

    @Install(to = "answerOptionsTable.moveDown", subject = "enabledRule")
    private boolean answerOptionsTableMoveDownEnabledRule() {
        SurveyAnswerOption answerOption = answerOptionsTable.getSingleSelected();

        return Objects.requireNonNull(answerOption).getDisplayOrder() < answerOptionsDc.getItems().size();
    }

    @Subscribe("answerOptionsTable.moveDown")
    public void onAnswerOptionsTableMoveDown(final Action.ActionPerformedEvent event) {
        SurveyAnswerOption answerOption = answerOptionsTable.getSingleSelected();
        Integer currentItemOrdinal = Objects.requireNonNull(answerOption).getDisplayOrder();
        Integer targetOrdinal = currentItemOrdinal + 1;
        List<SurveyAnswerOption> surveyAnswerOptions = answerOptionsDc.getMutableItems();
        for (SurveyAnswerOption surveyAnswerOption : surveyAnswerOptions) {
            if (surveyAnswerOption.getDisplayOrder().equals(targetOrdinal)) {
                surveyAnswerOption.setDisplayOrder(currentItemOrdinal);
            }
        }
        answerOption.setDisplayOrder(targetOrdinal);
        sortTable();
    }

    private void showHideAnswerOptions() {
        answerOptionsBox.setVisible(getEditedEntity().getAnswerType() != null &&
                (getEditedEntity().getAnswerType().equals(SurveyAnswerTypeEnum.SINGLE_ANSWER_MULTIPLE_CHOICE) ||
                        getEditedEntity().getAnswerType().equals(SurveyAnswerTypeEnum.MULTIPLE_ANSWER_MULTIPLE_CHOICE)));
    }

    private void sortTable() {
        answerOptionsTable.sort("displayOrder", Table.SortDirection.ASCENDING);
    }

    public void setDisplayOrder(Integer displayOrder) {
        this.displayOrder = displayOrder;
    }
}